Feature: create an IUK
  As a Tails developer,
  when I have prepared a new release,
  I want to build an IUK from the previous Tails release to the new one.

  Background:
    Given a usable temporary directory

  Scenario: create an IUK
    Given an old ISO image
    And a new ISO image
    When I create an IUK
    Then the "system.tar" archive in the saved IUK should contain a SquashFS diff
    And the "boot.tar.bz2" archive should contain an upgraded syslinux configuration
    And all files in the saved IUK should belong to owner 0 and group 0
    And all files in the "boot.tar.bz2" archive should belong to owner 0 and group 0
    And all files in the "system.tar" archive should belong to owner 0 and group 0

  Scenario: create an IUK using existing SquashFS diff file
    Given an old ISO image
    And a new ISO image
    And the "Tails_i386_0.11.1_to_0.11.2.squashfs" empty SquashFS diff
    When I create an IUK
    Then the saved IUK should contain a "FORMAT" file
    And the "FORMAT" file in the saved IUK should contain "1"
    And the saved IUK should contain a "control.yml" file
    And the saved IUK should contain a "system.tar" file
    And the "system.tar" archive in the saved IUK should contain "live/Tails_i386_0.11.1_to_0.11.2.squashfs"
    And the saved IUK should contain a "boot.tar.bz2" file

  Scenario: create an IUK using existing tarballs
    Given an old ISO image
    And a new ISO image
    And the "system.tar" archive that contains file "whatever1.file"
    And the "boot.tar.bz2" compressed archive that contains file "whatever2.file"
    When I create an IUK
    Then the saved IUK should contain a "FORMAT" file
    And the "FORMAT" file in the saved IUK should contain "1"
    And the saved IUK should contain a "control.yml" file
    And the saved IUK should contain a "system.tar" file
    And the "system.tar" archive in the saved IUK should contain "whatever1.file"
    And the saved IUK should contain a "boot.tar.bz2" file

  Scenario: create an IUK with deleted files in a monitored directory of the ISO filesystem
    Given an old ISO image that contains file "live/a"
    And a new ISO image that does not contain file "live/a"
    When I create an IUK
    Then the delete_files list should contain "live/a"

  Scenario: create an IUK with deleted files in a non-monitored directory of the ISO filesystem
    Given an old ISO image that contains file "whatever.dir/a"
    And a new ISO image that does not contain file "whatever.dir/a"
    When I create an IUK
    Then the delete_files list should not contain "whatever.dir/a"

  Scenario: create an IUK from identical ISO images
    Given two identical ISO images
    When I create an IUK
    Then the delete_files list should be empty
    And the "boot.tar.bz2" archive should contain an upgraded syslinux configuration

  Scenario: create an IUK when the kernel was not upgraded
    Given two ISO images that contain the same set of kernels
    When I create an IUK
    Then the "system.tar" archive in the saved IUK should not contain "live/vmlinuz"
    Then the "system.tar" archive in the saved IUK should not contain "live/initrd.img"
    Then the "system.tar" archive in the saved IUK should not contain "live/vmlinuz2"
    Then the "system.tar" archive in the saved IUK should not contain "live/initrd2.img"
    And the "boot.tar.bz2" archive should contain an upgraded syslinux configuration

  Scenario: create an IUK when the kernel was upgraded
    Given two ISO images when the kernel was upgraded
    When I create an IUK
    Then the "system.tar" archive in the saved IUK should contain "live/vmlinuz"
    Then the "system.tar" archive in the saved IUK should contain "live/initrd.img"
    And the "system.tar" archive in the saved IUK should not contain "filesystem.squashfs"
    And the "system.tar" archive in the saved IUK should not contain "filesyst.squ"
    And the "boot.tar.bz2" archive should contain an upgraded syslinux configuration

  Scenario: create an IUK when a new kernel was added
    Given two ISO images when a new kernel was added
    When I create an IUK
    Then the "system.tar" archive in the saved IUK should not contain "live/vmlinuz"
    Then the "system.tar" archive in the saved IUK should not contain "live/initrd.img"
    Then the "system.tar" archive in the saved IUK should contain "live/vmlinuz2"
    Then the "system.tar" archive in the saved IUK should contain "live/initrd2.img"
    And the "boot.tar.bz2" archive should contain an upgraded syslinux configuration

  Scenario: create an IUK when new files have appeared in filesystem.squashfs
    Given an old ISO image whose filesystem.squashfs does not contain file "A"
    And a new ISO image whose filesystem.squashfs contains file "A"
    When I create an IUK
    Then the saved IUK should contain a SquashFS that contains file "A"

  Scenario: create an IUK when files have disappeared from filesystem.squashfs
    Given an old ISO image whose filesystem.squashfs contains file "A"
    And a new ISO image whose filesystem.squashfs does not contains file "A"
    When I create an IUK
    Then the saved IUK should contain a SquashFS that deletes file "A"

  Scenario: create an IUK when files have been upgraded in filesystem.squashfs
    Given an old ISO image whose filesystem.squashfs contains file "A" modified at 1333333333
    And a new ISO image whose filesystem.squashfs contains file "A" modified at 1336666666
    When I create an IUK
    Then the saved IUK should contain a SquashFS that contains file "A" modified at 1336666666

  Scenario: create an IUK when the bootloader configuration was not upgraded
    Given two ISO images that contain the same bootloader configuration
    When I create an IUK
    Then the saved IUK should contain a "boot.tar.bz2" file
    And the "boot.tar.bz2" file in the saved IUK is compressed with bzip2
    And the "boot.tar.bz2" archive should contain an upgraded syslinux configuration

  Scenario: create an IUK when the bootloader configuration was upgraded
    Given two ISO images that do not contain the same bootloader configuration
    When I create an IUK
    Then the saved IUK should contain the new bootloader configuration
    And the "boot.tar.bz2" archive should contain an upgraded syslinux configuration

  Scenario: create an IUK without passing a product name

  Scenario: create an IUK whithout passing an old version number

  Scenario: create an IUK whithout passing a new version number

  Scenario: create an IUK whithout passing a build target
