Feature: install an IUK
  As a Tails user,
  Given the Tails upgrader proposes me to install an IUK that was downloaded and verified,
  When I accept,
  Then I want to get as a result either an upgraded system or an error message.

  Background:
    Given a 64MB Tails boot device with a blank MBR

  Scenario: attempt to install an IUK whose format version is not supported
    Given a "test1.iuk" IUK whose format version is not supported
    When I attempt to install the "test1.iuk" IUK
    Then it should fail
    And I should be told "Unsupported format"
    And the temporary directory on the system partition should be empty
    And the last line of the modules file should be "filesystem.squashfs"

  Scenario: attempt to install an IUK whose format version is not an integer
    Given a "test1.iuk" IUK whose format version cannot be determined
    When I attempt to install the "test1.iuk" IUK
    Then it should fail
    And I should be told "Unsupported format"
    And the temporary directory on the system partition should be empty
    And the last line of the modules file should be "filesystem.squashfs"

  Scenario: attempt to install an IUK that has no FORMAT file
    Given a "test1.iuk" IUK that has no FORMAT file
    When I attempt to install the "test1.iuk" IUK
    Then it should fail
    And I should be told "Can't read : No such file or directory"
    And the temporary directory on the system partition should be empty
    And the last line of the modules file should be "filesystem.squashfs"

  Scenario: install an IUK that contains no SquashFS
    Given a "test1.iuk" IUK that contains 0 SquashFS
    And a running Tails that has no IUK installed
    When I install the "test1.iuk" IUK
    Then it should succeed
    And the system partition should contain 1 SquashFS
    And the temporary directory on the system partition should be empty
    And the last line of the modules file should be "filesystem.squashfs"

  Scenario: install an IUK that contains one SquashFS
    Given a "test1.iuk" IUK that contains 1 SquashFS
    And a running Tails that has no IUK installed
    When I install the "test1.iuk" IUK
    Then it should succeed
    And the system partition should contain 2 SquashFS
    And the temporary directory on the system partition should be empty
    And the last line of the modules file should be "1.squashfs"

  Scenario: install an IUK that contains two SquashFS
    Given a "test1.iuk" IUK that contains 2 SquashFS
    And a running Tails that has no IUK installed
    When I install the "test1.iuk" IUK
    Then it should succeed
    And the system partition should contain 3 SquashFS
    And the temporary directory on the system partition should be empty
    And the last line of the modules file should be "2.squashfs"

  Scenario: install an IUK that contains no archive
    Given a "test1.iuk" IUK that contains no archive
    When I install the "test1.iuk" IUK
    Then it should succeed
    And the system partition should not contain file "in_archive_a"
    And the temporary directory on the system partition should be empty
    And the last line of the modules file should be "filesystem.squashfs"

  Scenario: install an IUK that contains one archive
    Given a "a.tar" archive that ships file "in_archive_a"
    And a "test1.iuk" IUK that contains archive "a.tar"
    When I install the "test1.iuk" IUK
    Then it should succeed
    And the system partition should contain file "in_archive_a"
    And the system partition should not contain file "in_archive_b"
    And the temporary directory on the system partition should be empty
    And the last line of the modules file should be "filesystem.squashfs"

  Scenario: install an IUK that contains two archives
    Given a "a.tar" archive that ships file "in_archive_a"
    And a "b.tar" archive that ships file "in_archive_b"
    And a "test1.iuk" IUK that contains archives "a.tar" and "b.tar"
    When I install the "test1.iuk" IUK
    Then it should succeed
    And the system partition should contain file "in_archive_a"
    And the system partition should contain file "in_archive_b"
    And the temporary directory on the system partition should be empty
    And the last line of the modules file should be "filesystem.squashfs"

  Scenario: install an IUK that contains two archives that ship the same file
    Given a "a.tar" archive that ships file "in_both_archives" with content "content1"
    Given a "b.tar" archive that ships file "in_both_archives" with content "content2"
    And a "test1.iuk" IUK that contains archives "a.tar" and "b.tar"
    When I install the "test1.iuk" IUK
    Then it should succeed
    And the system partition should contain file "in_both_archives" with content "content2"
    And the temporary directory on the system partition should be empty
    And the last line of the modules file should be "filesystem.squashfs"

  Scenario: install an IUK that should delete some files
    Given a "test1.iuk" IUK that should delete files "a, non_existent, b/c"
    And a system partition that contains file "a"
    And a system partition that contains file "b/c"
    And a system partition that contains file "d"
    When I install the "test1.iuk" IUK
    Then it should succeed
    And the system partition should not contain file "a"
    And the system partition should not contain file "b/c"
    And the system partition should contain file "d"
    And the temporary directory on the system partition should be empty
    And the last line of the modules file should be "filesystem.squashfs"

  Scenario: install an IUK a second time in a row
    Given a "a.tar" archive that ships file "in_archive_a"
    And a "test1.iuk" IUK that contains archive "a.tar"
    When I install the "test1.iuk" IUK
    And I install the "test1.iuk" IUK
    Then it should succeed
    And the system partition should contain file "in_archive_a"
    And the temporary directory on the system partition should be empty
    And the last line of the modules file should be "filesystem.squashfs"

  Scenario: install multiple IUK in a row
    Given a "a.tar" archive that ships file "in_archive_a"
    And a "test1.iuk" IUK that contains archive "a.tar"
    And a "b.tar" archive that ships file "in_archive_b"
    And a "test2.iuk" IUK that contains archive "b.tar"
    And a "test3.iuk" IUK that contains 1 SquashFS
    And a running Tails that has no IUK installed
    When I install the "test1.iuk" IUK
    And I install the "test2.iuk" IUK
    And I install the "test3.iuk" IUK
    Then it should succeed
    And the system partition should contain file "in_archive_a"
    And the system partition should contain file "in_archive_b"
    And the temporary directory on the system partition should be empty
    And the last line of the modules file should be "1.squashfs"

  Scenario: attempt to install an IUK while there is too little space available on the live medium
    Given a "a.tar" archive that ships file "in_archive_a"
    And a "b.tar" archive that ships the 80MB file "in_archive_b"
    And a "test1.iuk" IUK that contains archives "a.tar" and "b.tar"
    When I install the "test1.iuk" IUK
    Then it should fail
    And the system partition should not contain file "in_archive_a"
    And the system partition should not contain file "in_archive_b"
    And the temporary directory on the system partition should be empty
    And the last line of the modules file should be "filesystem.squashfs"

  @syslinux
  Scenario: install an IUK that ships a syslinux binary and MBR
    Given a "a.tar" archive that ships file "utils/linux/syslinux" copied from "/usr/bin/syslinux"
    Given a "b.tar" archive that ships file "utils/mbr/mbr.bin" copied from "/usr/lib/SYSLINUX/gptmbr.bin"
    And a "test1.iuk" IUK that contains archives "a.tar" and "b.tar"
    When I install the "test1.iuk" IUK
    Then it should succeed
    And the system partition should contain file "utils/linux/syslinux"
    And the system partition should contain file "utils/mbr/mbr.bin"
    And the system partition should contain file "syslinux/ldlinux.sys"
    And the "syslinux/ldlinux.sys" file in the system partition has been modified less than 1 minute ago
    And the MBR is the new syslinux' one

  # Scenario: attempt to install an IUK when running in "toram" mode
  #   Given a "test1.iuk" IUK
  #   And a Tails running in "toram" mode
  #   When I attempt to install the "test1.iuk" IUK
  #   Then it should fail
  #   And I should be told "The device Tails is running from cannot be found. Maybe you used the `toram' option?"
  #   And the temporary directory on the system partition should be empty
  #   And the last line of the modules file should be "filesystem.squashfs"

  # Scenario: attempt to install an IUK when running Tails from DVD
  #   Given a "test1.iuk" IUK
  #   And a Tails running from DVD
  #   When I attempt to install the "test1.iuk" IUK
  #   Then it should fail
  #   And I should be told "Tails is running from an optical device"
  #   And the temporary directory on the system partition should be empty
  #   And the last line of the modules file should be "filesystem.squashfs"

  # Scenario: attempt to install an IUK when running Tails from a USB stick installed the old "cat" way
  #   Given a "test1.iuk" IUK
  #   And a Tails running from a USB stick installed the old "cat" way
  #   When I attempt to install the "test1.iuk" IUK
  #   Then it should fail
  #   And I should be told "Tails is running from a device that was not created using Tails USB installer."
  #   And the temporary directory on the system partition should be empty
  #   And the last line of the modules file should be "filesystem.squashfs"
