use Test::Spec;

use 5.10.1;
use Data::Dumper;
use File::Temp qw{tempdir tempfile};
use Path::Class;
use Tails::IUK::Utils qw{extract_file_from_iso};
use Test::Fatal qw{dies_ok};

my @genisoimage_opts = qw{--quiet -J -l -cache-inodes -allow-multidot};
my @genisoimage = ('genisoimage', @genisoimage_opts);

describe 'The extract_file_from_iso function' => sub {
    describe 'when run on a non-existing ISO' => sub {
        it 'throws an exception' => sub {
            dies_ok { extract_file_from_iso('bla', 'non-existing.iso') };
        };
    };
    describe 'when asked to extract a non-existing file from an ISO' => sub {
        my $iso;
        before sub {
            my $tempdir = tempdir(CLEANUP => 1);
            $iso = file($tempdir, 'test.iso')->stringify;
            my $iso_tempdir = tempdir(CLEANUP => 1);
            system(@genisoimage, "-o", $iso, $iso_tempdir);
        };
        it 'throws an exception' => sub {
            dies_ok { extract_file_from_iso('non-existing.file', $iso) };
        };
    };
    describe 'when asked to extract an existing file from an ISO' => sub {
        my $iso;
        my $known_content = "known content";
        before sub {
            my $tempdir = tempdir(CLEANUP => 1);
            $iso = file($tempdir, 'test.iso')->stringify;
            my $iso_tempdir = tempdir(CLEANUP => 1);
            my $fh = file($iso_tempdir, 'vmlinuz')->openw;
            print $fh $known_content;
            close $fh;
            system(@genisoimage, "-o", $iso, $iso_tempdir);
        };
        it 'returns the file content' => sub {
            is(extract_file_from_iso('vmlinuz', $iso), $known_content);
        };
    };
};

runtests unless caller;
